-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------


-----------------------------------------------------------
-- Award Functions
-- These are the functions that allocate the DKP to one or many as needed
--
function RDKP_GetAwardPriveledges()
	-- Return true if this is the person who is the RL
	-- Only the RL/Officer In Charge should be doing this.
	-- Get Current Status in the raid
	-- Get Current Rank.
	-- figure that if player CanEditOfficerNote() then they are sufficient rank w/i the guild to be an officer w/ Raid point priv. 
	RDKP_Debug("GetAwardPriveledges");
	-- For right now, only the raid leader can post current Raid Awards.  
	-- This needs to be expanded for Guild-Wide updates.
	if IsRaidLeader() then
		return true;
	end
	return nil;
end

function RDKP_AwardDKPForKill(szType)
	-- A kill has occured, update all of the current raid members in the zone w/ DKP for this kill.
	RDKP_Debug("RDKP_AwardDKPForKill()")
	if RDKP_GetAwardPriveledges() then
		-- Move forward w/ allocating the DKP to each member.
		for i = 1, GetNumRaidMembers(), 1 do
			-- Get the info for who's in the raid.
			local szName, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(raidIndex);
			-- Make sure the character was there at the time of the kill:
			RDKP_Debug(szZone .. " == " .. GetRealZoneText() )
			if ( szZone == GetRealZoneText() ) then
				local szTrans = RDKP_Build_Transaction(szName, RDKP_TABLE_DKP_VALUE[szType], 0);
				local bResults = RDKP_Raid_SendInfo(szTrans);
				if ( not bResults ) then
					-- Problem 
					RDKP_Debug("DKP Not Awarded for " .. szName);
				end
			else
				RDKP_Debug(szName .. " not in Zone");
			end
		end
	end
end

-- Not sure how we're going to do this one.
function RDKP_AwardDKPForTime()
	-- Use Chronos for this functionality:
	-- Chronos.scheduleRepeating()
	-- Start repeating either at zone in while in raid, or upon trigger by RL.
	-- Every hour, DKP should be awarded.  
	-- We could set this up so that we award 1 pt every so many minutes based on a 60/DKPperHour value.
	if RDKP_GetAwardPriveledges() then
		-- Move forward w/ allocating the DKP
		for i = 1, GetNumRaidMembers(), 1 do
			-- Get the info for who's in the raid.
			local szName, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(raidIndex);
			local szTrans = RDKP_Build_Transaction(szName, iDKPTime, 0);
			local bResults = RDKP_Raid_SendInfo(szTrans);
			if ( not bResults ) then
				-- Problem 
				RDKP_Debug("DKP Not Awarded for " .. szName);
			end
		end
	end 
end

function RDKP_AwardGRForLoot(szLoot)
	-- Post kill, someone gets the loot.  
	-- If the loot is at a certain level, then the reciever needs to be credited for it
	-- Also need to track for shards here.
	-- If a shard is created w/ the loot that was just received, then the GR needs to be removed.
	-- Code stolen from CTRT
	if RDKP_GetAwardPriveledges() then
		local sPlayer, sLink;
		local iStart, iEnd, sPlayerName, sItem = string.find(szLoot, "([^%s]+) receives loot: (.+)%.");
		if ( sPlayerName ) then
			sPlayer = sPlayerName;
			sLink = sItem;
		else
			local iStart, iEnd, sItem = string.find(szLoot, "You receive loot: (.+)%.");
			if ( sItem ) then
				sPlayer = UnitName("player");
				sLink = sItem;
			end
		end
		-- Now that we have the gear, we need to get it's value
		if ( sLink and sPlayer ) then
			local sColor, sItem, sName, tTooltip = CT_RaidTracker_GetItemInfo(sLink);
			-- Need to check for shards at this point.  If there was a shard created, 
			-- then we need to refund the last transaction.
			local iGR = RDKP_GR_VALUE[sColor][GetRealZoneText()];
			if ( RDKP_Table_DEMats[sName] ) then
				-- This assumes that Blue gear drops Blue Shards/Dust.
				-- The real solution is to get the value of the previous trasaction for this character and return the reciprical amount.
				iGR = -iGR;
			end
			local szTrans = RDKP_Build_Transaction(sPlayer, 0, iGR);
			RDKP_Raid_SendInfo(szTrans)
		end
	end
end


function RDKP_Build_Transaction_Adjustment(szPlayer, iDKP, iGR)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	-- These adjustments should be unique to themselves.
	local szTransactionKey = RDKP_Build_RaidKeyID();
	local szTransaction = nil;
	for i in {szTransactionKey, 1, szPlayer, iDKP, iGR} do
		szTransaction = szTransaction .. RDKP_SZ_FIELD_DELIMITER .. i;
	end
	return szTransaction;
end


function RDKP_Build_Transaction(szPlayer, iDKP, iGR)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	RDKP_TransactionID = RDKP_TransactionID + 1;
	local szTransaction = nil;
	for i in {RDKP_RaidID, RDKP_TransactionID, szPlayer, iDKP, iGR} do
		szTransaction = szTransaction .. RDKP_SZ_FIELD_DELIMITER .. i;
	end
	return szTransaction;
end

function RDKP_Build_Historic_Transaction(szRaidID, iTransID, szPlayer, iDKP, iGR)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the guild.
	-- szRaidID = RaidID of the old raid.
	-- iTransID = Transaction ID for that raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	RDKP_TransactionID = RDKP_TransactionID + 1;
	local szTransaction = nil;
	for i in {szRaidID, szPlayer, szPlayer, iDKP, iGR} do
		szTransaction = szTransaction .. RDKP_SZ_FIELD_DELIMITER .. i;
	end
	return szTransaction;
end

function RDKP_Process_Transaction_Raid(szCaptured)
	RDKP_Debug("RDKP_Process_Transaction_Raid()");
	szPattern = "(" .. RDKP_RaidID .. ")" .. RDKP_SZ_FIELD_DELIMITER .. "(%d+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(%d+)" .. RDKP_SZ_FIELD_DELIMITER .. "(%d+)"
	local iStart, iEnd, szRaidID, iTransactionID, szPlayer, iDKP, iGR = string.find(szCaptured,  szPattern );
	return RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, iDKP, iGR);
end

function RDKP_Process_Transaction_Historic(szCaptured)
	RDKP_Debug("RDKP_Process_Transaction_Historic()");
	szPattern = "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(%d+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(%d+)" .. RDKP_SZ_FIELD_DELIMITER .. "(%d+)"
	local iStart, iEnd, szRaidID, iTransactionID, szPlayer, iDKP, iGR = string.find(szCaptured,  szPattern );
	return RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, iDKP, iGR);
end

function RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, iDKP, iGR)
	RDKP_Debug("RDKP_Process_Transaction()");
	-- Record the transaction.
	RDKP_Table_Raid_Register[szRaidID] = {
		[iTransactionID] =	{
			["player"] = szPlayer, 
			["DKP"] = iDKP,
			["GR"] = iGR
		}
	}
	-- At the time of the Transaction Processing, we should update the summaries.
	RDKP_UpdateSummarywithTransaction(szPlayer, iDKP, iGR)
	return true;
end

function RDKP_UpdateSummarywithTransaction(szPlayer, iDKP, iGR)
	-- Key point here though, is making sure that the master list is already as up-to-date as possible.
	-- Though argueably, if the summary isn't up to date, it will be once I get all the raid-data available onto the local list.
	-- The Player updated here also should be the Main Player, not necessarily the immediate player participating in the raid.
	local szMainCharacter = RDKP_Table_CharacterMap[szPlayer]["szName"];
	RDKP_Table_MasterPlayers[szMainCharacter]["DKP"] = RDKP_Table_MasterPlayers[szMainCharacter]["DKP"] + iDKP;
	RDKP_Table_MasterPlayers[szMainCharacter]["GR"] = RDKP_Table_MasterPlayers[szMainCharacter]["GR"] + iGR;
	RDKP_Table_MasterPlayers[szMainCharacter]["PRI"] =  RDKP_GetPriority(szMainCharacter);
end
